#include <HT1632.h>
#include <font_5x4.h>
#include <font_8x4.h>
#include <images.h>
#include <Wire.h>
#include <ds3231.h>
#include <DHT.h>

// Temperature and Humidity Sensor (DHT11) Settings
#define DHTPIN 2
#define DHTTYPE DHT11

// LED Matrix Pins
#define CSPIN 9
#define WRPIN 10
#define DATAPIN 11

// Space in ASCII is 32
#define PADDING 32

#define DOT 67
#define PERCENTAGE 37
#define COLON 58
#define FORWARDSLASH 47

char time_char[8];
char day_char[8];
char temp_char[9];
char hum_char[9];

struct ts t;
DHT dht(DHTPIN, DHTTYPE);


void setup()
{
  HT1632.begin(CSPIN, WRPIN, DATAPIN);

  Wire.begin();
  DS3231_init(DS3231_INTCN);
  dht.begin();
}

void loop()
{
  // This delay is added cause the DHT11 needs 2sec to give reliable values
  delay(2000);

  // Get the current time and date
  DS3231_get(&t);

  // Extract the time digits
  int lh = ((t.hour / 10) % 10);
  int rh = (t.hour % 10);
  int lm = ((t.min / 10) % 10);
  int rm = (t.min % 10);

  // Extract the date digits
  int ldt = ((t.mday / 10) % 10);
  int rdt = (t.mday % 10);
  int lmt = ((t.mon / 10) % 10);
  int rmt = (t.mon % 10);
  int y3d = ((t.year / 10) % 10);
  int y4d = ((t.year % 10));
  int y2d = (((t.year / 10) / 10) % 10);
  int y1d = ((((t.year / 10) / 10) / 10) % 10);

  // Fill in the time and date array with some padding
  time_char[0] = char(lh) + 48;
  time_char[1] = char(rh) + 48;
  time_char[2] = COLON;
  time_char[3] = char(lm) + 48;
  time_char[4] = char(rm) + 48;
  time_char[5] = PADDING;
  time_char[6] = PADDING;
  time_char[7] = PADDING;

  day_char[0] = char(ldt) + 48;
  day_char[1] = char(rdt) + 48;
  day_char[2] = FORWARDSLASH;
  day_char[3] = char(lmt) + 48;
  day_char[4] = char(rmt) + 48;
  day_char[5] = PADDING;
  day_char[6] = PADDING;

  // Read temp and humidity
  int hum = dht.readHumidity();
  int temp = dht.readTemperature();

  // Extract the temp and humidity digits
  int ltemp = ((temp / 10) % 10);
  int rtemp = (temp % 10);
  int lhum = ((hum / 10) % 10);
  int rhum = (hum % 10);

  // Fill in the temp and hum array with padding
  temp_char[0] = PADDING;
  temp_char[1] = PADDING;
  temp_char[2] = char(ltemp) + 48;
  temp_char[3] = char(rtemp) + 48;
  temp_char[4] = DOT;
  temp_char[5] = PADDING;
  temp_char[6] = PADDING;
  temp_char[7] = PADDING;
  temp_char[8] = PADDING;
  temp_char[9] = PADDING;

  hum_char[0] = char(lhum) + 48;
  hum_char[1] = char(rhum) + 48;
  hum_char[2] = PERCENTAGE;
  hum_char[3] = PADDING;
  hum_char[4] = PADDING;
  hum_char[5] = PADDING;
  hum_char[6] = PADDING;

  // Every 30 sec show the date, temp and humidity
  if (int(t.sec) == 30 || int(t.sec) == 0)
  {
    draw_image(day_char, 5);
    delay(3000);
    draw_image(temp_char, 5);
    delay(3000);
    draw_image(hum_char, 9);
    delay(5000);
  }
  else
  {
    draw_image(time_char, 5);
  }
}

void draw_image(char *imageToDraw, int startFrom)
{
  HT1632.clear();

  HT1632.drawText(imageToDraw, startFrom, 0, FONT_8X4, FONT_8X4_END, FONT_8X4_HEIGHT);

  HT1632.render();
}
